/*
 * FILE --- llp.hh
 *
 * PURPOSE --- Header file for a class that implements a Limited Lookahead Policy (LLP)
 * controller. The user specifies the dimension of the state-space, the dimension of the
 * control space, the lookahead horizon (i.e., the number of steps to lookahead), and
 * the total number of possible control inputs. The number of possible control inputs
 * needs to be finite so that the optimization is duable. Morover, the user has to 
 * provide the dynamics of the underlying control system, the cost function based on
 * which the optimization is performed, and all of the possible control inputs so that
 * proper optimization is performed. Given the current system state the algorithm 
 * generates all the possible future states within the lookahed horizon. Then it returns 
 * the control value of the first control input of the path with the smalles cost.
 *
 */

// Prevent Multiple Inclusion
#ifndef LLP_HH
#define LLP_HH

// Include Files
#include <stdlib.h>
#include "node_list.hh"

// Local Definitions
#define LLP_NO_ERROR 0
#define LLP_MEM_ERROR 1
#define LLP_INVD_CTRL -1

class LLP
{
public:
  
  // Constructor
  LLP(int _plant_order, int _ctrl_order, 
      int _lookahead_horizon, int _num_diff_ctrl);
  
  // Destructor
  virtual ~LLP();
  
  // Finds the cost of applying the control _control at state _state
  // It should contain the implementation of the cost function as 
  // cost = J(x,u), where x is the state and u is the control input.
  // The user provides this function, in general, by overloading it
  // in a derived class
  virtual double find_cost(const double *_state, const double *_control) {return 0;};
  
  // This function should contain the dynamics of the system in the form
  // x(k+1) = F(x(k),u(k)), where x(k) is the state and u(k) is the control 
  // input at time k. The next state x(k+1) is calculated within the function 
  // and returned within the argument _next_state. 
  // The user provides this function, in general, by overloading it
  // in a derived class
  virtual void dynamics(const double *_state, const double *_control, double *_next_state) {};
  
  // Set the all possible control inputs
  void set_ctrl_inputs(const double **_ctrl_inputs = NULL);
  
  // Set the specified control input
  void set_ctrl_input(int _input_num, const double *_ctrl_input = NULL);
  
  // Find the best control input for the current state
  void find_ctrl(double *_ctrl);
  
  double *plant_state; // Current system states
  double **ctrl_inputs; // Control inputs for the plant

  int llp_err;  // Variable to hold the error condition
  
protected:
  int plant_order; // Dimension of the state vector
  int ctrl_order; // Dimension of the control vector
  int lookahead_horizon; // Num of steps to lookahead
  int num_diff_ctrl; // Num of different controls
  
  // Pointers to possible future nodes and list of nodes
  // Conveniance variables 
  open_list *current_list;
  open_list *next_list;
  open_list *temp_list;
  open_node *current_node;
  open_node *next_node;
  open_node *temp_node;
  
private:
  // Allocate/free memory for the gains
  void allocate_memory(void);
  void free_memory(void);

  double *trial_state; // Trial system state
  double *next_state; // Next system state
  
  int num_valid_ctrl; // Num of valid controls for given state
  

};

#endif // LLP_HH
